﻿using System.Collections.Generic;
using System.Linq;

namespace System
{
    /// <summary>
    /// Supports cloning, which creates a new instance of a class with the same value as an existing instance.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public interface ICloneable<T> : ICloneable
    {
        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        new T Clone();
    }

    /// <summary>
    /// Support cloning of enumerable types that inherit ICloneable.
    /// </summary>
    public static class CloneEnumerable
    {
        public static IEnumerable<T> Clone<T>(this IEnumerable<T> enumerable) where T : ICloneable<T>
        {
            return enumerable.Select(i => i.Clone());
        }
    }
}
